# -*- coding: cp1252 -*-
# (C) by Fred Bayer <fred@bayerf.de>
#
# A GUI program for controlling to the F4LW linguistic sculpture.

from Tkinter import *
from tkFileDialog import *
from tkMessageBox import *
import os
import time
import thread
import shelve

import words
from f4lw import openSerial, uploadFile, uploadMem, uploadRTC
import astro

class F4LWWindow(Tk):
  """Main window class"""
  def __init__(self, parent):
    Tk.__init__(self)
    self.app = parent

    # Configure GUI layout
    for i in range(4):
      self.rowconfigure(i, pad=10, weight=1)
    self.rowconfigure(2, pad=10, weight=5)
    self.columnconfigure(0, pad=10, weight=0)
    self.columnconfigure(1, pad=10, weight=1)
    self.columnconfigure(2, pad=10, weight=0)

    # Create control variables
    self.romFileName = StringVar()
    self.romTag = StringVar()
    self.longitude = StringVar()
    self.latitude = StringVar()
    self.inclsun = IntVar()
    self.language = StringVar()
    self.termOnline = IntVar()
    self.numWords = StringVar()
    self.numWords.set("N/A")

    # Labels in leftmost column
    self.wordLabel = Label(self, text="Word mode", background="Red")
    self.clockLabel = Label(self, text="Clock mode", background="Green")
    self.termLabel = Label(self, text="Terminal mode", background="Yellow")
    self.msgLabel = Label(self, text="Message mode", background="Blue")

    # Word mode subframe
    wp = self.wordPane = Frame(self)
    try:
      self.romFileName.set(self.app.shelve["romfile"])
      self.romTag.set(self.app.shelve["romtag"])
      self.language.set(self.app.shelve["language"])
    except KeyError:
      self.language.set("en")
      
    wp.label1 = Label(wp, text="Text/ROM file")
    wp.label2 = Label(wp, text="ROM tag")
    wp.srcFile = Entry(wp, width=40, textvariable=self.romFileName)
    wp.romTag = Entry(wp, width=6, textvariable=self.romTag)
    wp.convertButton = Button(wp, text="Create ROM", command=self.createRom)
    wp.label1.grid(column=0, row=0, sticky=W)
    wp.label2.grid(column=0, row=1, sticky=W)
    wp.srcFile.grid(column=1, row=0, sticky=W)
    wp.romTag.grid(column=1, row=1, sticky=W)
    wp.convertButton.grid(column=2, row=0, rowspan=2)
    wp.srcFile.bind("<Double-Button-1>", self.selectFile)
    wp.langSelE = Radiobutton(wp, text="English", value="en", variable=self.language)
    wp.langSelG = Radiobutton(wp, text="German", value="de", variable=self.language)
    wp.langSelE.grid(column=1, row=2, sticky=W)
    wp.langSelG.grid(column=1, row=3, sticky=W)
    wp.wordInfo = Label(wp, textvariable=self.numWords)
    wp.wordInfo.grid(column=2, row=2, rowspan=2)

    # Clock mode subframe
    cp = self.clockPane = Frame(self)
    try:
      self.longitude.set(self.app.shelve["longitude"])
      self.latitude.set(self.app.shelve["latitude"])
      self.inclsun.set(self.app.shelve["inclsun"])
    except KeyError:
      self.longitude.set("0.0")
      self.latitude.set("0.0")
      self.inclsun.set(0)
      
    cp.label1 = Label(cp, text="Latitude")
    cp.label2 = Label(cp, text="Longitude")
    cp.inclSunCheck = Checkbutton(cp, text="Include sunrise/set", variable=self.inclsun)
    cp.latField = Entry(cp, width=15, justify=RIGHT, textvariable=self.latitude)
    cp.longField = Entry(cp, width=15, justify=RIGHT, textvariable=self.longitude)
    cp.label1.grid(column=0, row=1, sticky=W)
    cp.label2.grid(column=0, row=2, sticky=W)
    cp.inclSunCheck.grid(column=1, row=0)
    cp.latField.grid(column=1, row=1)
    cp.longField.grid(column=1, row=2)

    # Terminal mode subframe
    tp = self.termPane = Frame(self)
    tp.terminal = Text(self, height=1, width=40)
    #tp.terminal.grid(sticky=N+E+S+W)

    # Message mode subframe
    mp = self.msgPane = Frame(self)
    self.messages = [StringVar() for i in range(10)]
    try:
      for i in range(10):
        self.messages[i].set(self.app.shelve["messages"][i])
    except KeyError:
      pass
    mp.textFields = [Entry(mp, width=6, textvariable=self.messages[i]) for i in range(10)]
    for i in range(9):
      mp.textFields[i].grid(column=i%3, row=i//3, padx=5, pady=5)
    mp.textFields[9].grid(column=1, row=3, padx=5, pady=5)  
      
    # Create upload buttons
    self.uploadRomButton = Button(self, text="Upload ROM", command=self.uploadRom)
    self.setClockButton  = Button(self, text="Set clock", command=self.setClock)
    self.terminalButton  = Checkbutton(self, text="Online", command=self.toggleTerminal, variable=self.termOnline)
    self.uploadMsgButton = Button(self, text="Upload messages", command=self.uploadMsg)

    # Put all titles, subframes and buttons into main window
    self.wordLabel.grid(column=0, row=0, sticky=N+S+E+W) 
    self.clockLabel.grid(column=0, row=1, sticky=N+S+E+W) 
    self.termLabel.grid(column=0, row=2, sticky=N+S+E+W) 
    self.msgLabel.grid(column=0, row=3, sticky=N+S+E+W)

    self.wordPane.grid(column=1, row=0)
    self.clockPane.grid(column=1, row=1, sticky="")
    self.termPane.terminal.grid(column=1, row=2, sticky=N+S+E+W)
    self.msgPane.grid(column=1, row=3)
    
    self.uploadRomButton.grid(column=2, row=0, padx=10, sticky=E+W)
    self.setClockButton.grid(column=2, row=1, padx=10, sticky=E+W)    
    self.terminalButton.grid(column=2, row=2, padx=10, sticky=E+W)    
    self.uploadMsgButton.grid(column=2, row=3, padx=10, sticky=E+W)    

    try:
      self.geometry(self.app.shelve["geometry"])
    except KeyError:
      pass

  def setClock(self):
    try:
      now = time.localtime()
      isun = self.inclsun.get()
      if isun:
        mem = astro.makeSunblock(now.tm_year,
                                 float(self.latitude.get()),
                                 float(self.longitude.get()))
      uploadRTC(astro.timeAsRtc(time.localtime(), isun))
      if isun:
        uploadMem(mem)
    except Exception, e:
      showerror("Upload error", "Error uploading clock/sun table:\n%s" % e)

  def uploadRom(self):
    memFile = os.path.splitext(self.romFileName.get())[0] + ".f4lw"
    try:
      uploadFile(memFile)
    except Exception, e:
      showerror("Upload error", "Error uploading EEPROM data:\n%s" % e)

  def createRom(self):
    name = self.romFileName.get()
    if os.path.isfile(name):
      tag = self.romTag.get()[0:4].ljust(4)
      lang = self.language.get()
      memFile = os.path.splitext(name)[0] + ".f4lw"
      thread.start_new_thread(self.makeRom, (name, memFile, tag, lang))
    else:
      showerror("Wrong file", "%s not found" % name)

  def uploadMsg(self):
    try:
      strings = [self.messages[i].get().encode("cp1252")[0:4].ljust(4) \
                 for i in [9]+range(9)]
      mem = "".join(strings).translate(words.langTrans[self.language.get()])
      uploadRTC(mem)
    except Exception, e:
      showerror("Upload error", "Error uploading messages:\n%s" % e)

  def toggleTerminal(self):
    if self.termOnline.get():
      self.uploadRomButton["state"] = DISABLED
      self.setClockButton["state"] = DISABLED
      self.uploadMsgButton["state"] = DISABLED
      self.termPane.terminal.bind("<Any-KeyPress>", self.sendChar)
      self.serial = openSerial(timeout=0)
      thread.start_new_thread(self.receiveHandler, ())
    else:
      self.uploadRomButton["state"] = NORMAL
      self.setClockButton["state"] = NORMAL
      self.uploadMsgButton["state"] = NORMAL
      self.termPane.terminal.unbind("<Any-KeyPress>")

  def sendChar(self, evt):
    if evt.keysym_num < 256:
      c = chr(evt.keysym_num).translate(words.langTrans[self.language.get()])
      self.serial.write(c)

  def receiveHandler(self):
    while self.termOnline.get():
      c = self.serial.read(1)
      if c:
        self.termPane.terminal.insert(INSERT, c)
    self.serial.close()

  def selectFile(self, args):
    basedir = os.path.dirname(self.romFileName.get())
    name = askopenfilename(initialdir=basedir)
    if name:
      self.romFileName.set(name)
      self.numWords.set("N/A")

  def makeRom(self, tFile, rFile, tag, lang):
    self["cursor"] = "watch"
    self.wordPane.convertButton["state"] = DISABLED
    words.setWordlist(tFile)
    chain = words.Chain(4)
    chain.savemem(rFile, tag, lang)
    self.numWords.set("%d words" % len(chain))
    self.wordPane.convertButton["state"] = NORMAL
    self["cursor"] = "arrow"
    self.bell()

  def saveOptions(self):
    self.app.shelve["geometry"] = self.geometry()
    self.app.shelve["romfile"] = self.romFileName.get()
    self.app.shelve["romtag"] = self.romTag.get()
    self.app.shelve["language"] = self.language.get()
    self.app.shelve["longitude"] = self.longitude.get()
    self.app.shelve["latitude"] = self.latitude.get()
    self.app.shelve["inclsun"] = self.inclsun.get()
    self.app.shelve["messages"] = [self.messages[i].get() for i in range(10)]
 
class F4LWApp(object):
  """Application class"""
  def __init__(self):
    self.shelve = shelve.open(os.path.expanduser("~/f4lw.options"))
    self.mainWindow = F4LWWindow(self)
    self.mainWindow.title("F4LW Gui V1.5 by Fred Bayer")
    self.mainWindow.protocol("WM_DELETE_WINDOW", self.quit)

  def run(self):
    self.mainWindow.mainloop()

  def quit(self):
    self.mainWindow.saveOptions()
    self.shelve.close()
    self.mainWindow.destroy()
    self.mainWindow.quit()

if __name__=="__main__":
  app = F4LWApp()
  app.run()
